//=============================================================================
// (c) Copyright 2015 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804DAConvert.c  v3.0
//  Depedency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
// Description: This sample code demonstrates the usage of UD function
//				DAQ0804DAConvert(). This sample code describes the usage
//				for MPEDAQ0804 board. Channel and 16-bit output code are prompted
//              from the user, then output to the board. Other settings are
//              controlled via the jumpers for this board.
//============================================================================
#include "DSCUD_demo_def.h"
#include "mpedaq0804.h"

// globals
ERRPARAMS errorParams; // structure for returning error code and error string
// var declarations
DSCB dscb = 0; // handle used to refer to the board
DSCCB dsccb;    // structure containing ISA board settings
DSCCBP dsccbp; // structure containing PCI board settings
BoardInfo *bi=NULL;
char input_buffer[20];

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DA CONVERSION AND OUTPUT
//
//=============================================================================

int main ( void )
{

	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
    char input_buffer[20];
    BYTE return_value=0;
	int channel;
	int range;
	int dasim = 0;
	double Expected = 0.0;
	unsigned long int DACode;
	float Vrefs[2]={2.5,5};
//	int temp;

   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
	memset( &dsccb,0,sizeof(DSCCB));
	 
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
	
	
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the port_under_testboard. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEDAQ0804 for the MPEDAQ0804 board
	//	   2. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================
	printf("\nMPE-DAQ0804 DA CONVERT FUNCTION DEMO\n");
	dsccbp.boardtype = DSC_MPEDAQ0804;
    dsccbp.pci_slot  =0;

	if ( (DAQ0804InitBoard (&dsccbp)!= DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf (  "DAQ0804InitBoard error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
	bi=DSCGetBoardInfo(dsccbp.boardnum);
	
	//=========================================================================
	// III. DA CONVERSION AND OUTPUT
	//
	//		DAQ0804DASetSettings() function configures the board based on the user 
	//		inputs of channel,range,dasim.DAQ0804DAConvert() function converts the 
	//		the sample and displays the expected voltage.Obtaining the DACode from
	//		the user and conversion are performed until the user presses 'q'. 
	//=========================================================================
	 do
	 {
		 printf ( "Enter Channel Number:(0 - 3)<default:0>:");	
		 fgets(input_buffer,20,stdin);
		 if ( input_buffer[0] == '\n' )
		 {
			channel = 0;
		 }
		 else
		 {
			sscanf (input_buffer, "%d", &intBuff );
		 	channel = intBuff;
		 }
		 

		 printf ( "Enter output range (0 = 0-2.5V,1 = 0-5V)<default:0>:");	
		 fgets(input_buffer,20,stdin);
		 if ( input_buffer[0] == '\n' )
		 {
			range = 0;
		 }
		 else
		 {
			sscanf (input_buffer, "%d", &intBuff );
			range = intBuff;
		 }
		
	
		dasim = 0;
		if ( DAQ0804DASetSettings ( bi,range,dasim ) != DE_NONE )
	    {
		    dscGetLastError(&errorParams);
		    printf ("DAQ0804DASetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		    return 0;
	    }

		 printf ( "Enter output code (0-65535)<default:32768>:");	
		 fgets(input_buffer,20,stdin);
		 if ( input_buffer[0] == '\n' )
		 {
			DACode = 32768;
		 }
		 else
		 {
			sscanf (input_buffer, "%ld", &DACode );
		 }
		
		Expected = (double)(DACode/65535.0)* Vrefs[range];
		printf("Ouptput code = %ld     Expected value:%lf\n",DACode,Expected);

		if( (DAQ0804DAConvert ( bi, channel, DACode ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ("DAQ0804DAConvert error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
		printf("Press ENTER to repeat or 'q' to quit:");


	 }while ( getchar() != 'q' );

	//=========================================================================
    // IV. CLEANUP
    //
    //     Cleanup any remnants left by the program and free the resources used
    //     by the driver.
    //
    //     STEPS TO FOLLOW:
    //
    //     1. free the driver resources
    //=========================================================================
	
	 dscFree ( );
	
	

	return 0;

} // end main()

