//=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804DAWave.c   v7.00
//
// Depedency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//
// MPEDAQ0804DAWave.c  v7.0
// Description:
//			Sample program that performs D/A wave form generation using
//	the on-board circuit and D/A buffer.
//=============================================================================
#include "DSCUD_demo_def.h"
#include "mpedaq0804.h"


// globals
ERRPARAMS errorParams; // structure for returning error code and error string
// var declarations
DSCB dscb = 0; // handle used to refer to the board
DSCCB dsccb;    // structure containing ISA board settings
DSCCBP dsccbp; // structure containing PCI board settings
int portConfiguration[6];//used to store the port directions
BoardInfo *bi=NULL;
char input_buffer[20];

DAQ0804WAVEFORM waveform;
BYTE key = 0;
#define COUNTER_0     0x00
#define WG_SRC_CTR0   1     /* Rising edges on Counter 0 output */
#define WG_SRC_CTR1   2     /* Rising edges on Counter 1 output */
//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. D/A WAVE FORM
//	    IV. Cleanup
//
//=============================================================================
#define PI 3.14159265
int main ( void )
{

	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
  	char input_buffer[20];
    BYTE return_value=0;
	int i=0;
	int index =0,intial_value=0,channel=0,sawtooth=0,range=0;
	//unsigned long frequency;	
	float frequency;


   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
	memset( &dsccb,0,sizeof(DSCCB));
	

  	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
		
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the port_under_testboard. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEDAQ0804 for the MPEDAQ0804 board
	//	   2. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================
	printf("MPE-DAQ-0804 Waveform demo \n");
    	dsccbp.boardtype = DSC_MPEDAQ0804;
	dsccbp.pci_slot  = 0;

	 
	if(DAQ0804InitBoard ( &dsccbp ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("DAQ0804InitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	bi = DSCGetBoardInfo ( dsccbp.boardnum ); 	

	
	//===========================================================================================================================================
	// III. D/A WAVE FORM 
	//
	// DAQ0804WaveformReset() function resets the pointer to the initial position of D/A buffer.DAQ0804WaveformConfig() function configures  
	// the board based on the user inputs of frames,clock source,frequency.DAQ0804DASetSettings() function configures the board based on the  
	// user inputs of channel,range.DAQ0804WaveformBufferLoad() function loads the D/A conversion value into the D/A buffer.DAQ0804WaveformStart()
	// function starts D/A conversions.DAQ0804WaveformInc() function does manual increment on a space bar key stroke from the user.
	// DAQ0804WaveformPause() function stops waveform.The application quits on a 'q' press from the user.
	//============================================================================================================================================
	
	do
	{
	
		printf("Enter DA-OutPut Channel Number (0-3 ) <Default =0>: ");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{
		  channel = 0;
		}
		else
		{
		  sscanf(input_buffer,"%d", &channel);
		}
		
		printf("Enter Range value (0-1  0 = 0-2.5V, 1 = 0-5V) <Default =0>: ");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{
		  range = 0;
		}
		else
		{
		  sscanf(input_buffer,"%d", &range);
		}
		
		printf("Enter waveform size (1-1023 ) <Default =500>: ");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{
		 waveform.Frames = 500;
		}
		else
		{
		  sscanf(input_buffer,"%d", &waveform.Frames);
		}
		
		printf("Enter Clock source (0-3 , 0 -Manual, 1-Counter 0 output, 2-Counter 1 output , 3-External trigger on DIO pin 20, Default =1 : ");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{
		 waveform.Clock =1;
		}
		else
		{
		  sscanf(input_buffer,"%d", &waveform.Clock);
		}
		
		printf("Select  waveform type (0-Sine wave  1-Sawtooth Wave) <Default =0>: ");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{
		  sawtooth = 0;
		}
		else
		{
		  sscanf(input_buffer,"%d", &sawtooth);
		}
		printf("Enter waveform frequency  <Default =100>: ");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{
		  frequency = 100;
		}
		else
		{
		  sscanf(input_buffer,"%f", &frequency);
		}	
		 
		waveform.FrameSize=1;

		printf("Enter waveform mode(0-1, 0-One shot 1-Repeat mode, default = 1: ");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{
		  waveform.Cycle =1;
		}
		else
		{
		  sscanf(input_buffer,"%d", &waveform.Cycle);
		}
		 
		 
		 frequency = waveform.Frames * frequency ;
		
		  if(frequency > 50000000)
		  {
			printf("Invalid frequency selected \n");
			return 0;
		  }
		waveform.Rate=frequency;


		 //reset the pointer to the D/A buffer
		if( DAQ0804WaveformReset(bi) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ("RMM1616WaveformReset error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
			
		if( DAQ0804WaveformConfig(bi, &waveform ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ( "RMM1616WaveformConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}

		//============================================================================
		// IV. D/A WAVE START
		//
		//     Program the D/A conversion code into D/A buffer and start D/A wave
		//     form generator
		//
		//=============================================================================
		
		 //Allocate the memory to hold waveform values
		  waveform.Waveform =(unsigned int*) malloc(waveform.Frames * sizeof(unsigned int));
		  if(waveform.Waveform == NULL)
		  {
			printf("Could not allocate memory to hold waveform values\n");
			return 0;
		  }
			
		  waveform.Channels=(int *) malloc(16 * sizeof(int *));
		  if(waveform.Channels == NULL)
		  {
			printf("Could not allocate memory to hold channel values\n");
			return 0;
		  }
		
		  if(sawtooth)
		  {
			intBuff= 0xFFFF/waveform.Frames;
			waveform.Waveform[0]=intial_value;
			waveform.Waveform[waveform.Frames-1]=intial_value;
			for(index=1;index <waveform.Frames-1;index++)
			{			
			  intial_value+=intBuff;
			  waveform.Waveform[index]=intial_value;
			}
		  }
		  else
		  {
				for(index=0;index <waveform.Frames;index++)
				{			
				waveform.Waveform[index]= (int) ((sin( index * (360.0/waveform.Frames) * PI/180) + 1) * ( 0xFFFF/2)) ;

				
				}
		  }
		 
			if ( DAQ0804DASetSettings ( bi,range, 0) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				printf ("DAQ0804DASetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
		  
		for( index=0;index<16;index++)
		 {
			waveform.Channels[index]= channel;
			

		 }

		//the following loads the D/A conversion value into the D/A buffer.
		if( DAQ0804WaveformBufferLoad(bi,&waveform) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ("DAQ0804WaveformBufferLoad error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
			
		//start D/A conversions
		printf("Starting DA wave form generator \n");
		if( DAQ0804WaveformStart(bi) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			printf ("DAQ0804WaveformStart error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
		if(waveform.Clock == 0)
		{
			printf("\nPress space bar key to do manual increment or Press 'q' to quit  \n");
			while(1)
			{
			
				 key =getch();
			
				  if (key == 32)
				  {
					DAQ0804WaveformInc(bi);
				  }
				  else if(key == 'q')
				  {
				
					break;
				  }
				
				   
			}	
			DAQ0804WaveformReset(bi);
		}
		else
		{
			printf("Press ENTER key to stop wave form Generator \n");
			fgets(input_buffer,20,stdin);
		
			DAQ0804WaveformPause(bi);
			DAQ0804WaveformReset(bi);
		}
		
		free(waveform.Waveform);
		free(waveform.Channels);
		
   		printf ("press Enter to repeat or 'q' to quit:");
   		fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );
	//=========================================================================
	// V. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );
	
	

	return 0;

} // end main()

