//=============================================================================
// (c) Copyright 2015 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804LED.c   v7.0
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//
// MPEDAQ0804LED.c  v7.0
// Description: The application toggles on board LED whenever space bar key pressed
//			
//	
//=============================================================================
#include "DSCUD_demo_def.h"
#include "mpedaq0804.h"


// globals
ERRPARAMS errorParams; // structure for returning error code and error string
// var declarations
DSCB dscb = 0; // handle used to refer to the board
DSCCB dsccb;    // structure containing ISA board settings
DSCCBP dsccbp; // structure containing PCI board settings
int portConfiguration[6];//used to store the port directions
BoardInfo *bi=NULL;
char input_buffer[20];
int intBuff = 0,LEDOn=1; // temp variables of size int

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. LED toggling
//	    IV. Clean-up
//
//=============================================================================

int main ( void )
{
	int temp = 0; // temporary storage
  	char input_buffer[20];
  	BYTE return_value=0;
	char err1[500],err2[500];
	int i=0,key=0;
	
   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
	memset( &dsccb,0,sizeof(DSCCB));
	

  	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
		
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the port_under_testboard. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEDAQ0804 for the MPEDAQ0804 board
	//	   2. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

    printf ( "\nMPE-DAQ-0804 LED Demo\n" );
	 
	dsccbp.boardtype = DSC_MPEDAQ0804;
	dsccbp.pci_slot  = 0;
		 
	if(DAQ0804InitBoard ( &dsccbp ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("DAQ0804InitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}
	bi = DSCGetBoardInfo ( dsccbp.boardnum ); 	
    //=========================================================================
	//III. LED toggling
	// STEPS TO FOLLOW:
	//  1. Press space bar to toggle the one board LED
	//
	//=========================================================================
	
	
	
	do
	{
		printf("\nPress space bar key to toggle LED or Press 'q' to quit  \n");
		while(1)
		{
			
			 key =getch();
			
			  if (key == 32)
			  {
				if(LEDOn)
				{
					DAQ0804LED(bi,0);
					LEDOn = 0;
				}
				else
				{
					DAQ0804LED(bi,1);
					LEDOn = 1;
				}
			  }
			  else if(key == 'q')
			  {
				
				break;
			  }
				
			   
		}	
	
   		printf ("press Enter to repeat or 'q' to quit:");
   		fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );
	//=========================================================================
	// V. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );
	
	printf("DSC LED function completed \n");

	return 0;

} // end main()
