///=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEDAQ0804UserInt.c  v7.00
//
// Depedency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				
// Description: This sample code demonstrates the usage of UD function
//				UserInt(),also 'userfun' is called each time when
//              a timer or DIO raises an interrupt.
//============================================================================

#include "DSCUD_demo_def.h"
int count =0;
void userfun (void *param)
{
   count ++;
}

// globals
ERRPARAMS errorParams; // structure for returning error code and error string
// var declarations
DSCCB dsccb; // structure containing board settings
DSCCBP dsccbp; // structure containing board settings
//new variables
#include "mpedaq0804.h"
BoardInfo *bi;
DAQ0804USERINT inter;
DAQ0804COUNTER Ctr ;
char input_buffer[20];
//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. UserInt Settings
//	    IV. UserInt Testing
//		 V. Cleanup
//
//=============================================================================
int main ( void )
{

	int intBuff = 0; // temp variables of size int
	float rate = 0; // temporary storage
   	char input_buffer[20];
	int port=0,dir=0;
	DWORD baseadd=0;
	int Bit=0,Edge=0;
	memset (&dsccbp, 0, sizeof(DSCCBP) );
	memset (&inter,0 , sizeof(inter) );
  
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the MPEDAQ0804 board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

		printf ( "\nMPE-DAQ-0804 UserInt FUNCTIONS DEMO\n" );

		dsccbp.boardtype = DSC_MPEDAQ0804;
		dsccbp.pci_slot  = 0;
		 
		if(DAQ0804InitBoard ( &dsccbp ) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("DAQ0804InitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		bi = DSCGetBoardInfo ( dsccbp.boardnum ); 	
  		//=========================================================================
		// III. USER INT INITIALIZATION and Run
		//
		//		Sets up the rate,interrupt source for MPEDAQ0804UserInterruptSet()
		//=========================================================================
		do
		{
			inter.IntFunc=userfun;
			inter.Mode=0;
			inter.Enable=1;
			printf("Enter Interrupt source (2-4;  2 = counter 2 output, 3 = counter 3 output, 4 = digital I/O, default =2 ) : ");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0] == '\n')
			{
				inter.Source=2;
			}
			else
			{
				sscanf(input_buffer,"%d",&inter.Source);
			}
			
			if(DAQ0804UserInterruptConfig(bi,&inter) !=DE_NONE)
			{          
				dscGetLastError ( &errorParams );
				printf ( "DAQ0804UserInterruptConfig error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return 0;
			}
			
			if(inter.Source == 4)
			{
				printf("Enter DIO bit number to drive interrupts(0-20, default =0 ) : ");
				fgets(input_buffer,20,stdin);
				if(input_buffer[0] == '\n')
				{
					Bit = 0;
				}
				else
				{
					sscanf(input_buffer,"%d",&Bit);
				}
				
				printf("Select Edge value to generate interrupts(0 = falling edge, 1 = rising edge, default =0) : ");
				fgets(input_buffer,20,stdin);
				if(input_buffer[0] == '\n')
				{
					Edge = 0;
				}
				else
				{
					sscanf(input_buffer,"%d",&Edge);
				}
				
				
			}
			
			if(DAQ0804UserInterruptRun(bi,inter.Source,Bit,Edge) !=DE_NONE)
			{
				dscGetLastError ( &errorParams );
				printf ( "DAQ0804UserInterruptRun error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return 0;
			}
		
			if(inter.Source !=4)
			{
				printf("Enter Interrupt source frequency rate (1-50MHZ) (default = 1000)");
				fgets(input_buffer,20,stdin);
				if(input_buffer[0] == '\n')
				{
					rate = 1000;
				}
				else
				{
					sscanf(input_buffer,"%f",&rate);
				}
				
				Ctr.Rate  = rate ;
				Ctr.CtrNo = inter.Source ;
				Ctr.CtrOutEn 	= 0 ;
				Ctr.CtrOutPol 	= 0 ;
				Ctr.ctrOutWidth = 0 ;
		
				if(DAQ0804CounterSetRate(bi,&Ctr) !=DE_NONE)
				{
					dscGetLastError ( &errorParams );
					printf ( "DAQ0804CounterSetRate error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
					return 0;
				}
			}
			else
			{
				
			   printf("Enter DIO bit %d  direction (0-input,1-output default:0)",Bit);
			   fgets(input_buffer,20,stdin);
			   if(input_buffer[0] == '\n')
			   {
					dir = 0;
			   }
			   else
			   {
				sscanf(input_buffer,"%d",&dir);
			   }

			   if(DAQ0804DIOConfig(bi,Bit,dir) !=DE_NONE)
			   {
					dscGetLastError ( &errorParams );
					printf ( "DAQ0804DIOConfig error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
					return 0;
			   }

			
			}
			
			printf("Press ENTER key to terminate the application \n");
			while( !kbhit())
			{
				dscSleep(1000);
				printf("Count value %d \r",count);
				fflush(stdout);
			}
	        
	        fgets ( input_buffer, 20, stdin );
		
			inter.Enable= 0;
			if(DAQ0804UserInterruptCancel(bi,inter.Source) !=DE_NONE)
			{
			  
				dscGetLastError ( &errorParams );
				printf ( "DAQ0804UserInterruptCancel error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return 0;
			}
			
			
   		printf ("press Enter to repeat or 'q' to quit:");
   		fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );

	//=========================================================================
	// V. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	dscFree ( );
	
	printf ( "\nDSCUserIntFunctions completed.\n" );

	return 0;

} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main ( );

	return 0;
}

#endif

